"""
Hatch build hook for building frontend assets.
"""

import os
import platform
import shlex
import shutil
import subprocess
from pathlib import Path

from hatchling.builders.hooks.plugin.interface import BuildHookInterface


class FrontendBuildHook(BuildHookInterface):
    """
    A Hatch build hook to build frontend assets using npm.
    This hook runs during the build process to ensure that the frontend assets are built and included in the package.
    """

    PLUGIN_NAME = "frontend"

    def initialize(self, version, build_data):
        """
        This hook builds frontend assets in the following scenarios:
        1. Building wheel distribution (python -m build --wheel)
        2. Installing from GitHub (pip install git+https://github.com/reata/sqllineage.git). pip will build sdist first,
           then install from sdist. sdist itself does not include built static files, same as source code repo.
           Installation from sdist will trigger this hook. (by building a wheel from sdist)
        """
        if "READTHEDOCS" in os.environ:
            return
        py_path = Path("sqllineage")
        static_folder = "build"
        static_path = py_path / static_folder
        js_path = Path("sqllineagejs")
        use_shell = True if platform.system() == "Windows" else False
        try:
            # install npm dependencies
            subprocess.check_call(
                shlex.split("npm install"), cwd=str(js_path), shell=use_shell
            )
            # build the frontend assets
            subprocess.check_call(
                shlex.split("npm run build"), cwd=str(js_path), shell=use_shell
            )
            # move the built assets to the Python package directory
            source_build_path = js_path / static_folder
            if static_path.exists():
                shutil.rmtree(str(static_path))
            shutil.move(str(source_build_path), str(static_path))
            # add all files in the static_path to build_data artifacts
            for file_path in static_path.rglob("*"):
                if file_path.is_file():
                    relative_path = file_path.relative_to(Path("."))
                    build_data.setdefault("artifacts", []).append(str(relative_path))
        except subprocess.CalledProcessError as e:
            raise RuntimeError(f"Frontend build failed: {e}") from e
        except Exception as e:
            raise RuntimeError(
                f"An unexpected error occurred during frontend build: {e}"
            ) from e



import pytest

from sqllineage.core.metadata_provider import MetaDataProvider
from sqllineage.utils.entities import ColumnQualifierTuple

from ...helpers import assert_column_lineage_equal, generate_metadata_providers

providers = generate_metadata_providers(
    {
        "ods.source_tab": ["day_id", "user_id", "name"],
        "ods.target_tab": ["day_no", "user_code", "user_name"],
    }
)


@pytest.mark.parametrize("provider", providers)
def test_metadata_target_column(provider: MetaDataProvider):
    sql = """insert into ods.target_tab select day_id as acct_id, user_id as xxx, name as yyy from ods.source_tab"""
    assert_column_lineage_equal(
        sql=sql,
        column_lineages=[
            (
                ColumnQualifierTuple("name", "ods.source_tab"),
                ColumnQualifierTuple("user_name", "ods.target_tab"),
            ),
            (
                ColumnQualifierTuple("day_id", "ods.source_tab"),
                ColumnQualifierTuple("day_no", "ods.target_tab"),
            ),
            (
                ColumnQualifierTuple("user_id", "ods.source_tab"),
                ColumnQualifierTuple("user_code", "ods.target_tab"),
            ),
        ],
        metadata_provider=provider,
        test_sqlparse=False,
    )


@pytest.mark.parametrize("provider", providers)
def test_metadata_target_column_cte(provider: MetaDataProvider):
    sql = """
INSERT INTO ods.target_tab
WITH cte_table AS (SELECT day_id as acct_id, user_id as xxx, name as yyy FROM ods.source_tab)
SELECT acct_id, xxx, yyy FROM cte_table"""
    assert_column_lineage_equal(
        sql=sql,
        column_lineages=[
            (
                ColumnQualifierTuple("user_id", "ods.source_tab"),
                ColumnQualifierTuple("user_code", "ods.target_tab"),
            ),
            (
                ColumnQualifierTuple("day_id", "ods.source_tab"),
                ColumnQualifierTuple("day_no", "ods.target_tab"),
            ),
            (
                ColumnQualifierTuple("name", "ods.source_tab"),
                ColumnQualifierTuple("user_name", "ods.target_tab"),
            ),
        ],
        metadata_provider=provider,
        test_sqlparse=False,
    )



import os.path
from pathlib import Path
from typing import Optional

from sqlalchemy import (
    Column as SQLAlchemyColumn,
)
from sqlalchemy import (
    Integer,
    MetaData,
    inspect,
    text,
)
from sqlalchemy import (
    Table as SQLAlchemyTable,
)

from sqllineage import SQLPARSE_DIALECT
from sqllineage.core.metadata.dummy import DummyMetaDataProvider
from sqllineage.core.metadata.sqlalchemy import SQLAlchemyMetaDataProvider
from sqllineage.core.metadata_provider import MetaDataProvider
from sqllineage.core.models import Column, Table
from sqllineage.runner import LineageRunner


def _assert_table_lineage(lr: LineageRunner, source_tables=None, target_tables=None):
    for _type, actual, expected in zip(
        ["Source", "Target"],
        [lr.source_tables, lr.target_tables],
        [source_tables, target_tables],
    ):
        actual = set(actual)
        expected = (
            set()
            if expected is None
            else {Table(t) if isinstance(t, str) else t for t in expected}
        )
        assert (
            actual == expected
        ), f"\n\tExpected {_type} Table: {expected}\n\tActual {_type} Table: {actual}"


def _assert_column_lineage(lr: LineageRunner, column_lineages=None):
    expected = set()
    if column_lineages:
        for src, tgt in column_lineages:
            src_col: Column = Column(src.column)
            if src.qualifier is not None:
                src_col.parent = Table(src.qualifier)
            tgt_col: Column = Column(tgt.column)
            tgt_col.parent = Table(tgt.qualifier)
            expected.add((src_col, tgt_col))
    actual = {(lineage[0], lineage[-1]) for lineage in set(lr.get_column_lineage())}

    assert (
        set(actual) == expected
    ), f"\n\tExpected Lineage: {expected}\n\tActual Lineage: {actual}"


def assert_table_lineage_equal(
    sql: str,
    source_tables=None,
    target_tables=None,
    dialect: str = "ansi",
    test_sqlfluff: bool = True,
    test_sqlparse: bool = True,
):
    lr = LineageRunner(sql, dialect=SQLPARSE_DIALECT)
    lr_sqlfluff = LineageRunner(sql, dialect=dialect)
    if test_sqlparse:
        _assert_table_lineage(lr, source_tables, target_tables)
    if test_sqlfluff:
        _assert_table_lineage(lr_sqlfluff, source_tables, target_tables)


def assert_column_lineage_equal(
    sql: str,
    column_lineages=None,
    dialect: str = "ansi",
    metadata_provider: Optional[MetaDataProvider] = None,
    test_sqlfluff: bool = True,
    test_sqlparse: bool = True,
):
    metadata_provider = (
        DummyMetaDataProvider() if metadata_provider is None else metadata_provider
    )
    lr = LineageRunner(
        sql, dialect=SQLPARSE_DIALECT, metadata_provider=metadata_provider
    )
    lr_sqlfluff = LineageRunner(
        sql, dialect=dialect, metadata_provider=metadata_provider
    )
    if test_sqlparse:
        _assert_column_lineage(lr, column_lineages)
    if test_sqlfluff:
        _assert_column_lineage(lr_sqlfluff, column_lineages)


def generate_metadata_providers(test_schemas) -> list[MetaDataProvider]:
    dummy_provider = DummyMetaDataProvider(test_schemas)

    sqlite3_sqlalchemy_provider = SQLAlchemyMetaDataProvider("sqlite:///:memory:")
    metadata = MetaData()
    for full_table_name, columns_names in test_schemas.items():
        schema, table = full_table_name.split(".")
        if schema not in ("main", "temp") and not inspect(
            sqlite3_sqlalchemy_provider.engine
        ).has_schema(schema):
            db_file_path = Path(os.path.dirname(__file__)).parent.joinpath(
                f"{schema}.db"
            )
            with sqlite3_sqlalchemy_provider.engine.connect() as conn:
                conn.execute(text(f"ATTACH DATABASE '{db_file_path}' AS '{schema}'"))
        SQLAlchemyTable(
            table,
            metadata,
            *[SQLAlchemyColumn(c, Integer) for c in columns_names],
            schema=schema,
        )
    metadata.create_all(bind=sqlite3_sqlalchemy_provider.engine)

    return [dummy_provider, sqlite3_sqlalchemy_provider]
